unit binary;
{

Author : Rob Jarzabek
last update: 2021.03.20
supports following eHouse variants: WiFi, One - RS485/422 (RM, HM, EM), LAN - Ethernet (ERM, CM/LM)
* contains binary Status Decoders for controllers mentioned above.
* UDP server (receiving uC statuses)
* TCP client (sending control commands) - change outputs/programs only
http://www.iSys.Pl/
http://inteligentny-dom.ehouse.pro/
http://www.eHouse.Pro/


eHouse System Controllers Status analyzer Delphi 5.0+, Pascal, Lazarus, Kylix
Changing from Delphi5 may require update code to new/different components available in IDE

One Controller Panel PC software
capture data from all eHouse controllers via UDP (WiFi, RS-485/422, Ethernet)
load and synchronize status to local memory cache matrix
display status of desired controller on panel
Buttons are active and send events via tcp
        - directly to IP controllers,
        - or indirectly for Non-IP via CM/LM, eHouse.Pro Server. eHouse.Exe, eHouseWiFi.exe, eHouse4Java servers
This code can be a template of any eHouse application (autonomous or simple GUI):
- windows control panel,
- communication server,
- communication gateway,
- log analizer
- Predictive Maintenance Template
- AI - Software Template
- BIM Template

This source code loacation:   http://www.iSys.Pl/download/delphi/2021/

Verified for version as of date 20121121
Verified for version as of date 20210320

File contains binary status frame (definition) received via UDP or TCP directly from eHouse controllers
}
//var         CURRENT_PGM:integer;

interface
procedure    get_hm_desc;
procedure set_rm(str:array of byte;TCP_INDEX_INC:integer);
procedure set_status_commmanager(statusb:array of byte);
procedure set_status_ethroommanager(statusb:array of byte);
procedure set_status_wifi(statusb:array of byte);
function calcindex(high,low:byte):integer;
const MAX_TEMP=6;
var    WENT_MODE:integer;       //binary coded ventilation mode from HM
boiler_alarm:boolean=false;   //boiler alarm above programmed alarm temperature
bonfire_stat:integer; //bonfire heating level 0-7 for binary mode
//0 - fire off
//1 - fire on / extinguishing => green level<temp jacket (green led blinking)
//2 - fire on => green level>=temp jacket>yellow level - histeressis (green led)
//3 - fire on => yellow level - histeressis>temp jacket>yellow level+ histeressis) (green + yellow leds)
//4 - fire on => yellow level +histeressis>temp jacket>red level - histeressis (yellow led)
//5 - unconditional => red level - histeressis < temp jacket < red level + histeressis (red+yellow leds)
//6 - unconditional => temp jacket > red level + histeressis     (red led constant)
//7 - unconditional => temp jacket > bonfire alarm (red led blinking) ALARM

bonfire_sensor_error:boolean=false;//bonfire sensors error too much different water jacket temperature between two sensors
recuperator_status:string;      // status of recuperation, ventilation, air heating
recu_winter:boolean=false;	//recuperator winter/summer mode heat exchanger enabled / disabled
recu_manual_amalva:boolean=true;  //recuperator auto/manual mode for amalva recuperator (auto managed by internal amalva controler, manual by eHouse HeatManager)
went_cooler:boolean=true;       //water cooler on/off for ventilation or recuperation
went_gwc:boolean=false;		//gwc (ground heat exchanger) servomotor on/off
went_aux_gwc_fan:boolean=false;		//auxiliary gwc vetilator / fan on/off for ground heat exchanger
bonfire_dgp:boolean=false;      //bonfire hot air distribution system enabled / disabled
ventilation_on:boolean=false;   //ventilation is currently working on/off
heater_pump:boolean=true;       //pump for heater for ventilation on / off
three_ways_cutoff:integer=0;    //three ways cutoff direction 0 off, 1 increasing level, -1 decreasing level)
solar_pump:boolean=false;	//solar pump on/off
boiler_on:boolean=false;        //boiler enabled / disabled
boiler_pump:boolean=false;	//boiler pump status on/off
boiler_fuel_out:boolean=false;   //boiler out of fuel aproximate (count hours of heating)
boiler_power_on:boolean=false; //boiler power supplay on/off;
boiler_supply_en:boolean=false; //boiler supplay enabled 
boiler_fuel_supply_override:boolean=false; //boiler fuel supplyier override (control by ehouse / internal boiler controller)
bonfire_pump:boolean=false;   //bonfire pump status on/off


heati:array[1..50] of string; //names taken from ehouse.exe HeatManager control panel
firsttime:boolean=true;       //first time execution flag for inicialization
RECU_MODE:integer;           //binary coded recuperation mode from HM
recu_speed:integer=0;     //recuperator speeed 0..3
recu_temperature:integer; //temperature level of recu 0..32
_recu_mode:string='';     //recuperation mode in text as on HEatManager control panel (ehouse.exe)
kom_level:integer=0;    //level for bonfire 0-7 the same as bonfire_stat
kom_stat:integer=0;
        went:string;    //ventilation text status as on HeatManager Control panel (ehouse.exe)
        koci,sola:string; //boiler,solar text status as on HeatManager Control panel (ehouse.exe)
        komi:string; //bonfire text status as on HeatManager Control panel (ehouse.exe)

const   _RECU_WINTER 			=$80;		//winter/summer lato zima 7 bit
        _RECU_MANUAL 			=$40;		//manual - auto bit 6
        _RECU_FULL_AUTO                 =$20;           //recu full auto mode pelna automatyka wentylacji
                                                        //lower bits 0-5 temperature - temperatura 5 lsb 0 do 30
//went mode
        _WENT_CHLODNICA 		=$80;		//cooler pump on for gwc / Wczenie chlodnicy w gwc Pompy
        _WENT_GWC			=$40;		//ground heat exchanger for gwc on off/ Wybr Czerpni GWC lub Poludniwej
        _WENT_WENTYLATOR_GWC	        =$20;		//auxiliary ground heat exchanger for gwc / Wczenie wentylatora wspomagajcego GWC
        _WENT_DGP                       =$10;           //DGP Fan on/off  / Wlczenie wentylatora DGP
        _WENT_UNCONDITIONAL_WENT        =$08;           //Unconditional Ventilation / Bezwarnunkowe wlaczenie wentylacji
        _WENT_CHLODZENIE                =$04;           //Cooling On/ off / chlodzenie

 const  STATUS_ADC		= 1+2;                 //byte indexes in status frame for binary mode - indeksy bajtw z danymi w ramce statusu
        STATUS_OUT  	        =17+2;
        STATUS_IN		=20+2;
        STATUS_INT		=21+2;
        STATUS_OUT25 	        =22+2;
        STATUS_LIGHT	        =23+2;
        STATUS_ZONE_PGM         =26+2;
        STATUS_PROGRAM          =27+2;
        STATUS_INPUTEXT_A_ACTIVE=28+2;
        STATUS_INPUTEXT_B_ACTIVE=32+2;
        STATUS_INPUTEXT_C_ACTIVE=36+2;
        STATUS_INPUTEXT_A 	=40+2;
        STATUS_INPUTEXT_B 	=50+2;
        STATUS_INPUTEXT_C	=60+2;
        STATUS_ADC_HEART		=1+2;
        STATUS_OUT_HEART  	        =33+2;
        STATUS_IN_HEART		        =36+2;
        STATUS_INT_HEART		=37+2;
        STATUS_LOG_HEART		=38+2;		//sumaconst
        STATUS_RAM_SIZE_HEART         =40+2;
        STATUS_ROM_SIZE_HEART         =42+2;		//suma 40
        HM_STATUS_PROGRAM             = 36 + 2;//hm current program
        HM_STATUS_KOMINEK             = 46 + 2;//hm status bonfire
        HM_STATUS_RECU                = 48 + 2;//hm status recu
        HM_WENT_MODE		      = 49 + 2; //hm vent mode
// status commmanager
//eHouse LAN
STATUS_EHOUSE1_DEVS		=0;			//eHouse RS485 via CM/LM index
STATUS_DIMM_ETH                =4;
STATUS_ADC_ETH			=72;			//adc 16 * 2B
STATUS_ADC_ETH_END		=STATUS_ADC_ETH+32;	//84	//104
STATUS_OUT_I2C			=STATUS_ADC_ETH_END;   //2 razy i2c razy 10 rej po 8  //max=160 wyjsc	ind 92	//112
STATUS_INPUTS_I2C		=STATUS_OUT_I2C+20;	//2 razy i2c razy 6 rej po 8	//max 96 wejsc	ind 104
STATUS_ALARM_I2C		=STATUS_INPUTS_I2C+12;	//--|---
STATUS_WARNING_I2C		=STATUS_ALARM_I2C+12;		//--|---
STATUS_MONITORING_I2C	        =STATUS_WARNING_I2C+12;	//--|---			//160
STATUS_PROGRAM_NR		=STATUS_MONITORING_I2C+12; //--|---
STATUS_ZONE_NR			=STATUS_PROGRAM_NR+1;
STATUS_ADC_PROGRAM		=STATUS_ZONE_NR+1;
STATUS_LIGHT_LEVEL		=STATUS_ADC_PROGRAM+2;		//3 dimmers - tak jest w html
STATUS_PROFILE_ERM              =180-1;
////////////////////////////////////////////////////////////////////////////////
////eHouse WiFi
OFFSET_WIFI_I=4;
STATUS_ADC_WIFI                 =0+OFFSET_WIFI_I;
STATUS_DIMM_WIFI                 =8+OFFSET_WIFI_I;
STATUS_OUT_WIFI                 =12+OFFSET_WIFI_I;
STATUS_INPUTS_WIFI              =13+OFFSET_WIFI_I;
STATUS_PROGRAM_NR_WIFI          =14+OFFSET_WIFI_I;
STATUS_ADC_PROGRAM_WIFI         =15+OFFSET_WIFI_I;
STATUS_MODE_WIFI=14+OFFSET_WIFI_I;
STATUS_MOD2_WIFI=15+OFFSET_WIFI_I;
STATUS_ADCL_WIFI=16+OFFSET_WIFI_I;
STATUS_ADCH_WIFI=24+OFFSET_WIFI_I;
STATUS_RSSI_WIFI=32+OFFSET_WIFI_I;
STATUS_QSI_WIFI=33+OFFSET_WIFI_I;
STATUS_SINCE_WIFI=34+OFFSET_WIFI_I;
STATUS_OUTS_COUNT		=50;	//?????


implementation
uses loganalizer,  ExtCtrls, StdCtrls,filectrl,sysutils,
  Windows;//, Messages, Classes, Graphics, Controls, Forms, Dialogs


/////////////////////////////////////////////////////////////////////////////////
//calculate temp from lm335 temperature sensor
function gettemplm(dtah,dtal:byte;calibration:integer;k:double;VCC:integer):double;
var temp:double;
var t:integer;
begin
t:=ord(dtah)*256+ord(dtal);
temp:=(t*VCC/(1023*k))+calibration/100;
t:=round(temp*10);
temp:=t/10;
result:=temp;
end;
//get absolute adc value 0..1023
/////////////////////////////////////////////////////////////////////////////////////////////////////
function calculate_adc_value(high,low:byte):double;
var tmp:integer;
begin
tmp:=ord(high);
tmp:=tmp and 3;
tmp:=tmp shl 8;
tmp:=tmp+ord(low);
//tmp:=tmp shr 6;
result:=(tmp*100)/1023;//ADC_QUANT_10;
end;
//////////////////////////////////////////////////////////
//get adc value of eHouse wifi dev
function calculate_adc_wifi_value(high,low:byte):double;
var tmp:integer;
begin
tmp:=ord(high);
tmp:=tmp and 3;
//tmp:=tmp shl 8;
tmp:=tmp+ord(low);
//tmp:=tmp shr 6;
result:=(tmp*100)/1023;//ADC_QUANT_10;
end;
/////////////////

////////////////////////////////////////////////////////////////////////////
//get light value in negative scale
/////////////////////////////////////////////////////////////////////////////////////
function getlight(dtah,dtal:byte):double;
var
dta:integer;
dtd:double;
begin
dta:=ord(dtah);
dta:=dta and 3;
dta:=dta shl 8;
dta:=dta+ord(dtal);
dta:=1023-dta;
dtd:=dta*1000;
dta:=round(dtd/1023);
result:=dta/10;
end;
///////////////////////////////////////////////////////////////////////////////////////
//update heatmanager states for RM array
procedure update_hm_temps(index:integer);
var k,ind:integer;
begin
ind:=0;
for k:=0 to TEMP_COUNT-1 do
        RM[index].temp[k]:=HADC[k];
  end;

////////////////////////////////////////////////////////////////////////////////////////
//read heatmanager description fields
procedure get_hm_desc;
var tf:textfile;
i:integer;
begin
if (not fileexists (remotepath+'panels\hm_desc.txt')) then
                        copyfile(Pchar(remotepath+''+inttostr(getuserdefaultLCID)+'.het'),Pchar(remotepath+'panels\hm_desc.txt'),false); //copy file for posibility of changing description for panels
if (not fileexists (remotepath+'panels\hm_desc.txt')) then
                        copyfile(Pchar(remotepath+'hm_desc.txt'),Pchar(remotepath+'panels\hm_desc.txt'),false); //copy file for posibility of changing description for panels

if fileexists(remotepath+'panels\hm_desc.txt') then
        begin
        assignfile(tf,remotepath+'panels\hm_desc.txt');
        reset(tf);
        for i:=1 to 50 do
                begin
                readln(tf,heati[i]);
                end;
        closefile(Tf);
        end;
end;
/////////////////////////////////////////////////////////////////////////////////
//set and fill RM array with current data decoded directly from binary status for eHouse1 Controllers
procedure set_rm(str:array of byte;        TCP_INDEX_INC:integer);
var     m,c,tmp:integer;
        offset:integer;
        dh,dl:integer;
        sms:string;
        statt:array[0..5] of integer;
        tmp1:integer;
        index:integer;
        i,k,l,a,b:integer;
        errors:integer;
        tff:file;
        tf1,tf:textfile;
        count:integer;
        st:string;
        sss:array [0..255] of char;
        statust:string;
        recu_stat:string;
        WENT_MODE,RECU_MODE:integer;
        externalmanageradr:boolean;
        additional:string;
        heartmanager:boolean;
label donotcheck;
begin
{if (firsttime) then
   begin
   get_hm_desc;
   end;
   firsttime:=false;}
additional:='';
heartmanager:=false;
externalmanageradr:=false;
errors:=0;
index:=0;
recu_stat:='';

for k:=0 to ROOMMANAGER_COUNT do
        begin
        if (rm[k].adrh=ord(str[0+TCP_INDEX_INC])) and (rm[k].adrl=ord(str[1+TCP_INDEX_INC])) then
                begin
                rm[k].datetime:=now;
                rm[k].getnewdata:=true;
                end;
        if ((rm[k].adrh=ord(str[0+TCP_INDEX_INC])) and (rm[k].adrl=ord(str[1+TCP_INDEX_INC]))) then
                begin
                index:=k;
                break;
                end;
        if rm[k].devname=null then exit;
        if length(rm[k].devname)<1 then exit;


        end;

heartmanager:=false;
if ((str[0+TCP_INDEX_INC]=byte(1)) and (str[1+TCP_INDEX_INC]=byte(1))) then
        heartmanager:=true;           //set heatmanager flag from address
if ((str[0+TCP_INDEX_INC]=byte(2)) and (str[1+TCP_INDEX_INC]=byte(1))) then
externalmanageradr:=true;     //set externalmanager flag from address - obsolete
TCP_INDEX_INC:=0;
st:='';
if (heartmanager) then          //HeatManager
    begin                       //Wentylation, Recuperation, Bonfire and other dedicated status
//            HM_STATUS_PROGRAM             = 36 + 2;//hm current program
//        HM_STATUS_KOMINEK             = 46 + 2;//hm status bonfire
//        HM_STATUS_RECU                = 48 + 2;//hm status recu
//        HM_WENT_MODE		      = 49 + 2; //hm vent mode

    WENT_MODE:= ord(str[51+TCP_INDEX_INC]);
    RECU_MODE:= ord(str[50+TCP_INDEX_INC]);
    rm[index].currentprogram:=ord(str[38+TCP_INDEX_INC]);
    if ((rm[index].currentprogram>=0) and (rm[index].currentprogram<24)) then
            rm[index].currentprogramname:={rm[index].}programnames[rm[index].currentprogram];
    b:=RECU_MODE AND $1f;
    c:=WENT_MODE AND 3;
    recu_STAT:=heati[1]+' '+inttostr(ord(c))+', T: '+inttostr(ord(b))+' C, '+heati[2]+' ';
    recu_speed:=c;
    recu_temperature:=b;
    if (RECU_MODE and _RECU_WINTER)     >0	then begin _recu_mode:=heati[3];recu_winter:=true;end
    else 					     begin _recu_mode:=heati[4];recu_winter:=false;end;
    if (RECU_MODE and _RECU_MANUAL)     >0	then begin _recu_mode:=_recu_mode+heati[5];recu_manual_amalva:=true;end
    else 					     begin _recu_mode:=_recu_mode+heati[6];recu_manual_amalva:=false;end;
    recu_stat:=recu_stat+_recu_mode;
    if ((WENT_MODE and _WENT_CHLODNICA)>0 )		then
        begin
        recu_stat:=recu_stat+heati[7];
        went_cooler:=true;
        end
    else
        begin
        went_cooler:=false;
        end;
    if ((WENT_MODE and _WENT_GWC)  >0   )	        then
        begin
        went_gwc:=true;
        recu_stat:=recu_stat+heati[9];
        end
    else
        begin
        went_gwc:=false;
        end;
    if ((WENT_MODE and _WENT_WENTYLATOR_GWC)>0)	        then
        begin
        went_aux_gwc_fan:=true;
        recu_stat:=recu_stat+heati[11];
        end
    else
        begin
        went_aux_gwc_fan:=false;
        end;
    if ((WENT_MODE and _WENT_DGP)>0)                   then
        begin
        bonfire_dgp:=true;
        recu_stat:=recu_stat+heati[13];
        end
    else
        begin
        bonfire_dgp:=false;
        end;
    recu_stat:=heati[16]+copy(recu_stat,1,length(recu_Stat)-2);
    b:=  ord(str[49+TCP_INDEX_INC]);
    c:= ord(str[47+TCP_INDEX_INC]);
    went:='';
    komi:='';
    koci:='';
    sola:='';
    if (b and 32)>0 then
        begin
        went:=heati[18];
        ventilation_on:=true;
        end
    else
        begin
        ventilation_on:=false;
        went:=heati[19];
        end;
    if (b and 16)>0 then
                begin
                went:=went+heati[20];
                heater_pump:=true;
                end
    else
                begin
                heater_pump:=false;
                end;
    if (b and 64)>0 then
                begin
                three_ways_cutoff:=1;
                went:=went+' ,+';
                end
    else three_ways_cutoff:=0;
    if (b and 128)>0 then
                begin
                went:=went+' ,-';
                three_ways_cutoff:=-1;
                end;
           if length(went)>0 then statust:=heati[22]+went+recu_stat;
    if (b and 4)>0 then
                begin
                solar_pump:=true;
                sola:=heati[18];
                end
        else
                begin
                solar_pump:=false;
                sola:=heati[19];
                end;
    a:=ord(str[48+TCP_INDEX_INC]);

    if (b and 8)>0 then
                begin
                boiler_on:=true;
                koci:=koci+heati[18];
                end
    else
                begin
                boiler_on:=false;
                koci:=koci+heati[19];
                end;
    if (b and 2)>0 then
                begin
                boiler_pump:=true;
                koci:=koci+heati[26];
                end
    else
                begin
                boiler_pump:=false;
                end;
    if (a and 16)>0 then
                begin
                boiler_fuel_out:=true;
                koci:=koci+heati[27];
                end
    else
                begin
                boiler_fuel_out:=false;
                end;
    if (a and 32)>0 then
                begin
                koci:=koci+' ,!!Alarm!!';
                boiler_alarm:=true;
                end
    else
                begin

                boiler_alarm:=false;
                end;

    if (a and 64)>0 then
                begin
                boiler_power_on:=true;
                koci:=koci+heati[30];
                end
    else
                begin
                boiler_power_on:=false;
                end;
    if (a and 128)>0 then
                begin
                boiler_supply_en:=true;
                koci:=koci+heati[32];
                end
    else
                begin
                boiler_supply_en:=false;
                end;
    if (c and 1)>0 then
                begin
                boiler_fuel_supply_override:=true;
                koci:=koci+heati[34];
                end
    else
                begin
                boiler_fuel_supply_override:=false;
                end;
    if length(koci)>0 then
                statust:=statust+heati[36]+koci;
    a:= (a and $07);
    case a of
        0:   begin komi:=komi+heati[19];bonfire_stat:=0;  end;
        1:   begin komi:=komi+'1';bonfire_stat:=1;end;
        2:   begin komi:=komi+'2';bonfire_stat:=2;end;
        3:   begin komi:=komi+'3';bonfire_stat:=3;end;
        4:   begin komi:=komi+'4';bonfire_stat:=4;end;
        5:   begin komi:=komi+'5';bonfire_stat:=5;end;
        6:   begin komi:=komi+'6';bonfire_stat:=6;end;
        7:   begin komi:=komi+'!!Alarm!!';bonfire_stat:=7;end;
    end;
    kom_level:=a;
    bonfire_pump:=false;
    if (b and 1)>0 then
                begin
                bonfire_pump:=true;
                komi:=komi+heati[37];
                end;
    if (a and 8)>0 then
                begin
                bonfire_sensor_error:=true;
                komi:=komi+heati[38];
                end
    else bonfire_sensor_error:=false;
    if length(komi)>0 then
                begin
                statust:=statust+heati[40]+komi
                end
    else
                begin
                end;
    if length(sola)>0 then  statust:=statust+heati[41]+sola;
    for i:=0 to 15 do                                           //heatManager ADC convertion
        begin
        HADC[i]:=gettemplm(str[STATUS_ADC_HEART+i*2+TCP_INDEX_INC],str[STATUS_ADC_HEART+i*2+1+TCP_INDEX_INC],-27315{rm[index].RCAL1},10,5000);//rm[index].VCC);
        st:=st+floattostr(HADC[i])+chr(9);
        rm[0].temp[i]:=HADC[i];
        end;
        update_hm_temps(index);
         st:=st+' '+statust;
    //end;
   bonfire_status:=komi;  //status of bonfire for HeatManager
   boiler_status:=koci;   //status of boiler for HM
   Solar_status:=sola;    //status of solar system
   recuperator_status:=went;// status of recuperation, ventilation, air heating
   loganalizer.recuperator_status:=went;
   end
   else                 //other then HeatManager
        begin
        rm[index].currentprogram:=ord(str[STATUS_PROGRAM+TCP_INDEX_INC]);
        if ((rm[index].currentprogram>=0) and (rm[index].currentprogram<24)) then
            rm[index].currentprogramname:={rm[index].}programnames[rm[index].currentprogram];

{       we do not support Input extenders any longer

         if (rm[index].extern) then
                begin
        rm[index].currentzone:=ord(str[STATUS_ZONE_PGM+TCP_INDEX_INC]);
        inextstat[1]:=#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0;
        inextstat[2]:=#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0;
        inextstat[3]:=#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0;
        inextstat[4]:=#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0;
        inextstat[5]:=#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0+#0;
        inextstat[1]:=str[STATUS_INPUTEXT_A+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+3+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+4+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_A+5+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+6+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+7+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+8+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A+9+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_A_ACTIVE+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A_ACTIVE+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A_ACTIVE+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_A_ACTIVE+3+TCP_INDEX_INC];
        inextstat[2]:=str[STATUS_INPUTEXT_B+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+3+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+4+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_B+5+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+6+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+7+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+8+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B+9+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_B_ACTIVE+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B_ACTIVE+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B_ACTIVE+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_B_ACTIVE+3+TCP_INDEX_INC];
        inextstat[3]:=str[STATUS_INPUTEXT_C+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+3+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+4+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_C+5+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+6+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+7+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+8+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C+9+TCP_INDEX_INC]+
                      str[STATUS_INPUTEXT_c_ACTIVE+TCP_INDEX_INC]+str[STATUS_INPUTEXT_c_ACTIVE+1+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C_ACTIVE+2+TCP_INDEX_INC]+str[STATUS_INPUTEXT_C_ACTIVE+3+TCP_INDEX_INC];
        saveinex;
               end;}
        for k:=0 to 8 do
                begin
                rm[index].adc[k]:=calculate_adc_value(str[2*k + STATUS_ADC+TCP_INDEX_INC],str[2*k + STATUS_ADC+1+TCP_INDEX_INC]);
                rm[index].LIGHT[k]:=getlight(str[2*k + STATUS_ADC+TCP_INDEX_INC],str[2*k + STATUS_ADC+1+TCP_INDEX_INC]);
                rm[index].TEMP[k]:=gettemplm(str[2*k + STATUS_ADC+0+TCP_INDEX_INC],str[2*k + STATUS_ADC+1+TCP_INDEX_INC],-27315,10,5000);//rm[index].RCAL1,10,rm[index].VCC);
                end;
end;


k:=0;
if not heartmanager then
        begin
        statt[k]:=ord(str[STATUS_OUT+0+TCP_INDEX_INC]);inc(k);
        statt[k]:=ord(str[STATUS_OUT+1+TCP_INDEX_INC]);inc(k);
        statt[k]:=ord(str[STATUS_OUT+2+TCP_INDEX_INC]);inc(k);
        statt[k]:=ord(str[STATUS_OUT25+0+TCP_INDEX_INC]);inc(k);

        tmp:=ord(str[STATUS_OUT+2+TCP_INDEX_INC]);
        tmp:=tmp shl 16;
        tmp1:=ord(str[STATUS_OUT+1+TCP_INDEX_INC]);
        tmp1:=tmp1 shl 8;
        tmp:=tmp+tmp1+ord(str[STATUS_OUT+TCP_INDEX_INC]);

        end
else
        begin
        statt[k]:=ord(str[STATUS_OUT_HEART+0+TCP_INDEX_INC]);inc(k);
        statt[k]:=ord(str[STATUS_OUT_HEART+1+TCP_INDEX_INC]);inc(k);
        statt[k]:=ord(str[STATUS_OUT_HEART+2+TCP_INDEX_INC]);inc(k);
        statt[k]:=0;//ord(str[STATUS_OUT25+0+TCP_INDEX_INC]);inc(k);

        tmp:=ord(str[STATUS_OUT_HEART+2+TCP_INDEX_INC]);
        tmp:=tmp shl 16;
        tmp1:=ord(str[STATUS_OUT_HEART+1+TCP_INDEX_INC]);
        tmp1:=tmp1 shl 8;
        tmp:=tmp+tmp1+ord(str[STATUS_OUT_HEART+TCP_INDEX_INC]);
//        rm[index].currentprogram:=8;//ord(str[STATUS_OUT_HEART+4+TCP_INDEX_INC]);
        end;
begin

begin                                   //set output states from binary data
k:=0;
//offset:=0;
for m:=0 to 4-1 do         //bajty                                                //wejscia zwykle max 40
        for i:=0 to 7 do //bity
                begin
                if ((ord(statt[m]) and (1 shl i))>0) then
                        begin
                        rm[index].outs[m*8+i]:=true;
                        end
                else
                        rm[index].outs[m*8+i]:=false;
                end;
end;
if (not heartmanager) then
begin
tmp:=ord(str[STATUS_INT+TCP_INDEX_INC]);        //set input states from binary data
offset:=0;
//stat[0]:=
m:=0;
        for i:=0 to 7 do //bity
                begin
                if ((ord(tmp) and (1 shl i))>0) then
                        begin
                        rm[index].ints[m*8+i]:=true;
                        end
                else
                        rm[index].ints[m*8+i]:=false;
                end;

tmp:=ord(str[STATUS_IN]);               //set input part 2 states from binary data
m:=0;
        for i:=0 to 7 do //bity
                begin
                if ((ord(tmp) and (1 shl i))>0) then
                        begin
                        rm[index].ints[m*8+i]:=true;
                        end
                else
                        rm[index].ints[m*8+i]:=false;
                end;

end;
end;
{Input extenders not supported any longer
if (rm[index].extern) then
        begin
        sms:=getalarmstate(1);
        sms:=sms+getalarmstate(2);
        sms:=sms+getalarmstate(3);

        if length(sms)>0 then
                begin
                writeln(tf,'Alarm: '+sms);
                communication.admem('SECU: '+sms);
                if alarmextender>0 then
                        begin
                        wasalarm:=true;
                        for i:=0 to smsreports.Count-1 do
                                begin
                                createsms(smsreports.strings[i],'Alarm: '+sms);
                                end;
                        end;
                for i:=0 to smsadmins.Count-1 do
                        begin
                        createsms(smsadmins.strings[i],'Alarm: '+sms);
                        end;
                end;
        end;
        }
   k:=1;

end;
////////////////////////////////////////////////////////////////////////////////
// set status of CommManager attached to eHouse1 controllers status
// or explicit i pure Ethernet Ehouse Version
procedure set_status_commmanager(statusb:array of byte);
var str:string;
m,k,l,i:integer;
offset:integer;
Alarms:string;
Warnings:string;
monitorings:string;
Actives:string;
Outputs:string;
ADCS:string;
rm:boolean;
begin
rm:=false;

//if statusb[1]=#0 then exit;      //comm manager status of e-house 1 was send first
                                // if not commmanager then exit
if (statusb[0]=255) then exit;  //eHouse.PRO we dont want it here
//if ((statusb[1]>byte(0)) and (statusb[1]<byte(100)) and (statusb[2]<byte(100))) then rm:=true;      //Ehouse1 controllers Manager status  //device address high 1-100
if ((statusb[1]>124) and (statusb[1]<130)) then exit;     //addresses reserved for CAN, RF, Aura, RFID, Satel
if ((
        (statusb[1]=byte(1)) or                                 //Heat Manager
        (statusb[1]=byte(2)) or                                 //External Manager eHouse 1
        (statusb[1]=byte(55))) and                              //RoomManager eHouse1
        (statusb[2]<byte(100)) and                              //are adressed from 1 except ERM,CM/LM, EPM,
        ((statusb[0]>byte(104)) and (statusb[0]<byte(113))      //109 size is RM, HM
        ))
        //larger status size might be eHouse1 via CM/LM
         then
                rm:=true;      //Ehouse1 controllers Manager status  //device address high 1-100
if (rm) then exit;      //we process RM, EM, HM elswere

if ( (not rm) and (statusb[2]<byte(250))) then exit;      //EthernetEhouseManager status  if device adress low >= 240 also commmanager explicit
//if (statusb[3]<>'s') then exit;
ethrm[commmanagerindex].getnewdata:=true;               //set new data capture flag

ethrm[commmanagerindex].datetime:=now;                  //set data capture for info


offset:=0;
for i:=0 to 15 do       //przetworniki - ADCs
        begin
        ethrm[commmanagerindex].adcs[i]:=calculate_adc_value(statusb[STATUS_ADC_ETH+offset+i*2] ,statusb[STATUS_ADC_ETH+offset+i*2+1]);
        ethrm[commmanagerindex].adcsind[i]:=calcindex(statusb[STATUS_ADC_ETH+offset+i*2] ,statusb[STATUS_ADC_ETH+offset+i*2+1]);
        ethrm[commmanagerindex].temps[i]:=gettemplm(statusb[STATUS_ADC_ETH+offset+i*2] ,statusb[STATUS_ADC_ETH+offset+i*2+1],-27315{rm[index].RCAL1},10,3300);
        ethrm[commmanagerindex].tempsMCP9701[i]:=calculate_MCP9701(ethrm[commmanagerindex].adcsind[i],ethrm[commmanagerindex].VCC,ethrm[commmanagerindex].MCP9701_Offset,ethrm[commmanagerindex].MCP9701x);
        ethrm[commmanagerindex].tempsMCP9700[i]:=calculate_MCP9700(ethrm[commmanagerindex].adcsind[i],ethrm[commmanagerindex].VCC,ethrm[commmanagerindex].MCP9700_Offset,ethrm[commmanagerindex].MCP9700x);

        end;
offset:=0;
for m:=0 to 19 do         //bajty                                                //wyjscia i2c 20 bytow * 8 max 160 wyjsc
        for i:=0 to 7 do //bity                                                 //i2c outputs max 160 outputs
                begin
                if ((ord(statusb[STATUS_OUT_I2C+offset+m]) and (1 shl i))>0)   then     //wlaczone wyjscie   / output on
                        begin
                        ethrm[commmanagerindex].xdouts[m*8+i]:=true;
                        end
                else
                        begin
                        ethrm[commmanagerindex].xdouts[m*8+i]:=false;
                        end;
                end;

offset:=0;
for m:=0 to 11 do         //bajty                                                //wejscia i2c zwykle max 96    // i2c digital inputs max 96
        for i:=0 to 7 do //bity
                begin
                if ((ord(statusb[STATUS_INPUTS_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[commmanagerindex].xdins[m*8+i]:=true;
                        end
                else
                        ethrm[commmanagerindex].xdins[m*8+i]:=false;

                if ((ord(statusb[STATUS_ALARM_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[commmanagerindex].xalarms[m*8+i]:=true;
                        end
                else ethrm[commmanagerindex].xalarms[m*8+i]:=false;

                if ((ord(statusb[STATUS_WARNING_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[commmanagerindex].xwarnings[m*8+i]:=true;
                        end
                else ethrm[commmanagerindex].xwarnings[m*8+i]:=false;

                if ((ord(statusb[STATUS_MONITORING_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[commmanagerindex].xmonitorings[m*8+i]:=true;
                        end
                else
                        ethrm[commmanagerindex].xmonitorings[m*8+i]:=false;





                end;
//set important information for cm: Current Zone, Current Program, Zone Name, Program Name, ADC program, AdC Programname
ethrm[commmanagerindex].currentprogram:=ord(statusb[STATUS_PROGRAM_NR]);

ethrm[commmanagerindex].currentzone:=ord(statusb[STATUS_ZONE_NR]);
ethrm[commmanagerindex].currentadcprogram:=ord(statusb[STATUS_ADC_PROGRAM]);
if ethrm[commmanagerindex].currentprogram<XPROGRAMS_COUNT then
        ethrm[commmanagerindex].currentprogramname:=ethrm[commmanagerindex].programsnames[ethrm[commmanagerindex].currentprogram];
if ethrm[commmanagerindex].currentzone<XZONES_COUNT then
        ethrm[commmanagerindex].currentzonename:=ethrm[commmanagerindex].zonenames[ethrm[commmanagerindex].currentzone];
if ethrm[commmanagerindex].currentadcprogram<XADC_PROGRAMS_COUNT then
        ethrm[commmanagerindex].currentadcprogramname:=ethrm[commmanagerindex].adcprogramsnames[ethrm[commmanagerindex].currentadcprogram];




end;
////////////////////////////////////////////////////////////////////////////////////
//
// Perform status of other ethernet controllers than CommManager
//
///////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//
// Perform status of other ethernet controllers than CommManager
//
///////////////////////////////////////////////////////////////////////////////////
procedure set_status_ethroommanager(statusb:array of byte);
var str:string;
m,k,l,i:integer;
offset:integer;
Alarms:string;
Warnings:string;
monitorings:string;
Actives:string;
Outputs:string;
ADCS:string;
ethroommanagerindex:integer;

begin
ethroommanagerindex:=-1;
//if ((statusb[1]>#0) and (statusb[1]<#100)) then exit;      //RoomManagers via commmanager status reserved
if ((statusb[2]<byte(100))) then exit;
if (statusb[2]>=byte(250)) then exit;        //commmanager exclusive status reserved
if (statusb[3]<>byte(ord('s'))) then exit;         //only status data perform

for i:=0 to ethernetdevscount do
        begin
        if ((ethrm[i].adrh=ord(statusb[1])) and (ethrm[i].adrl=ord(statusb[2]))) then
                begin
                ethroommanagerindex:=i;
                break;
                end;

        end;

if ethroommanagerindex<0 then exit;
//if commManagerIndex=ethroommanagerindex then exit;
ethrm[ethroommanagerindex].getnewdata:=true;
ethrm[ethroommanagerindex].datetime:=now;

offset:=0;

for i:=0 to 15 do       //przetworniki - adcs
        begin
        ethrm[ethroommanagerindex].adcs[i]:=calculate_adc_value((statusb[STATUS_ADC_ETH+offset+i*2]) ,statusb[STATUS_ADC_ETH+offset+i*2+1]);
        ethrm[ethroommanagerindex].adcsind[i]:=calcindex((statusb[STATUS_ADC_ETH+offset+i*2]) ,statusb[STATUS_ADC_ETH+offset+i*2+1]);
        ethrm[ethroommanagerindex].temps[i]:=gettemplm((statusb[STATUS_ADC_ETH+offset+i*2]) ,statusb[STATUS_ADC_ETH+offset+i*2+1],-27315{rm[index].RCAL1},10,3300);
        ethrm[ethroommanagerindex].tempsMCP9701[i]:=calculate_MCP9701(ethrm[ethroommanagerindex].adcsind[i],ethrm[ethroommanagerindex].VCC,ethrm[ethroommanagerindex].MCP9701_Offset,ethrm[ethroommanagerindex].MCP9701x);
        ethrm[ethroommanagerindex].tempsMCP9700[i]:=calculate_MCP9700(ethrm[ethroommanagerindex].adcsind[i],ethrm[ethroommanagerindex].VCC,ethrm[ethroommanagerindex].MCP9700_Offset,ethrm[ethroommanagerindex].MCP9700x);


 //      if pos('@',ethrm[ethroommanagerindex].adcsnames[i])=0 then
//                ADCS:=ADCS+#9+ethrm[ethroommanagerindex].adcsnames[i]+':'+#9+GetStringValue(ethroommanagerindex,ethrm[ethroommanagerindex].adcsind[i],i)+eoln;
        end;

offset:=0;

ethrm[ethroommanagerindex].dimmers[0]:= round((ord(statusb[STATUS_DIMM_ETH+offset+0])*100)/255);///10;
ethrm[ethroommanagerindex].dimmers[1]:= round((ord(statusb[STATUS_DIMM_ETH+offset+1])*100)/255);///10;
ethrm[ethroommanagerindex].dimmers[2]:= round((ord(statusb[STATUS_DIMM_ETH+offset+2])*100)/255);///10;
for m:=0 to 20 do         //bajty                                                //normal digital outputs
        for i:=0 to 7 do //bity
                begin
                if ((ord(statusb[STATUS_OUT_I2C+offset+m]) and (1 shl i))>0)   then     //wlaczone wyjscie
                        begin
//                        Outputs:=Outputs+#9+ethrm[ethroommanagerindex].xdoutsnames[m*8+i]+eoln;
                        ethrm[ethroommanagerindex].xdouts[m*8+i]:=true;
                        end
                else
                        begin
                        ethrm[ethroommanagerindex].xdouts[m*8+i]:=false;
                        end;
                end;

//                end;
offset:=0;
for m:=0 to 11 do         //bajty                                                //normal digital inputs
        for i:=0 to 7 do //bity
                begin
                if ((ord(statusb[STATUS_INPUTS_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[ethroommanagerindex].xdins[m*8+i]:=true;
                        end
                else
                        ethrm[ethroommanagerindex].xdins[m*8+i]:=false;

                if ((ord(statusb[STATUS_ALARM_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[ethroommanagerindex].xalarms[m*8+i]:=true;
                        end
                else
                        ethrm[ethroommanagerindex].xalarms[m*8+i]:=false;

                if ((ord(statusb[STATUS_WARNING_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[ethroommanagerindex].xwarnings[m*8+i]:=true;
                        end
                else
                        ethrm[ethroommanagerindex].xwarnings[m*8+i]:=false;

                if ((ord(statusb[STATUS_MONITORING_I2C+offset+m]) and (1 shl i))>0) then
                        begin
                        ethrm[ethroommanagerindex].xmonitorings[m*8+i]:=true;
                        end
                else
                        ethrm[ethroommanagerindex].xmonitorings[m*8+i]:=false;

                end;
//set some important information as program, adc program, program name, adc program name
//ethrm[ethroommanagerindex].currentprogram:=ord(statusb[STATUS_PROGRAM_NR+offset]);
ethrm[ethroommanagerindex].currentprogram:=ord(statusb[STATUS_PROFILE_ERM+offset]);
ethrm[ethroommanagerindex].currentzone:=ord(statusb[STATUS_ZONE_NR+offset]);
ethrm[ethroommanagerindex].currentadcprogram:=ord(statusb[STATUS_ADC_PROGRAM+offset]);

if ethrm[ethroommanagerindex].currentprogram<XPROGRAMS_COUNT then
        ethrm[ethroommanagerindex].currentprogramname:=ethrm[ethroommanagerindex].programsnames[ethrm[ethroommanagerindex].currentprogram];
if ethrm[ethroommanagerindex].currentzone<XZONES_COUNT then
        ethrm[ethroommanagerindex].currentzonename:=ethrm[ethroommanagerindex].zonenames[ethrm[ethroommanagerindex].currentzone];
if ethrm[ethroommanagerindex].currentadcprogram<XADC_PROGRAMS_COUNT then
        ethrm[ethroommanagerindex].currentadcprogramname:=ethrm[ethroommanagerindex].adcprogramsnames[ethrm[ethroommanagerindex].currentadcprogram];

end;



//////////////////////////////////////////////////////////////////////////////////

procedure set_status_wifi(statusb:array of byte);
var str:string;
m,k,l,i:integer;
high,low:byte;
//offset:integer;
Alarms:string;
Warnings:string;
monitorings:string;
Actives:string;
Outputs:string;
ADCS:string;
wifiindex:integer;

begin
wifiindex:=-1;
//if ((statusb[1]>#0) and (statusb[1]<#100)) then exit;      //RoomManagers via commmanager status reserved
if ((statusb[0]>byte(50))) then exit;     //we ignore non wifi controllers by status size
if (statusb[3]<>byte(ord('s'))) then exit;         //only status data to perform
for i:=0 to wifidevscount do
        begin
        if ((wc[i].adrh=ord(statusb[1])) and (wc[i].adrl=ord(statusb[2]))) then
                begin
                wifiindex:=i;
                break;
                end;

        end;

if wifiindex<0 then exit;
wcc[wifiindex].getnewdata:=true;
wcc[wifiindex].datetime:=now;
for i:=0 to 3 do       //ADC - physically only one
        begin
        high:=statusb[STATUS_ADC_WIFI+i*2];
        low:=statusb[STATUS_ADC_WIFI+i*2+1];
        wcc[wifiindex].adcs[i]:=calculate_adc_value(high ,low);
        wcc[wifiindex].adcsind[i]:=calcindex(high ,low);
        wcc[wifiindex].temps[i]:=gettemplm(high ,low,-27315,10,1000);
        wcc[wifiindex].tempsMCP9701[i]:=calculate_MCP9701(wcc[wifiindex].adcsind[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9701_Offset,wcc[wifiindex].MCP9701x);
        wcc[wifiindex].tempsMCP9700[i]:=calculate_MCP9700(wcc[wifiindex].adcsind[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9700_Offset,wcc[wifiindex].MCP9700x);

        high:=statusb[STATUS_ADCL_WIFI+i*2];
        low:=statusb[STATUS_ADCL_WIFI+i*2+1];

        wcc[wifiindex].adcsL[i]:=calculate_adc_value(high ,low);
        wcc[wifiindex].adcsindL[i]:=calcindex(high ,low);
        wcc[wifiindex].tempsL[i]:=gettemplm(high ,low,-27315,10,1000);
        wcc[wifiindex].tempsMCP9701L[i]:=calculate_MCP9701(wcc[wifiindex].adcsindL[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9701_Offset,wcc[wifiindex].MCP9701x);
        wcc[wifiindex].tempsMCP9700L[i]:=calculate_MCP9700(wcc[wifiindex].adcsindL[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9700_Offset,wcc[wifiindex].MCP9700x);

        high:=statusb[STATUS_ADCH_WIFI+i*2];
        low:=statusb[STATUS_ADCH_WIFI+i*2+1];

        wcc[wifiindex].adcsH[i]:=calculate_adc_value(high ,low);
        wcc[wifiindex].adcsindH[i]:=calcindex(high ,low);
        wcc[wifiindex].tempsH[i]:=gettemplm(high ,low,-27315,10,1000);
        wcc[wifiindex].tempsMCP9701H[i]:=calculate_MCP9701(wcc[wifiindex].adcsindH[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9701_Offset,wcc[wifiindex].MCP9701x);
        wcc[wifiindex].tempsMCP9700H[i]:=calculate_MCP9700(wcc[wifiindex].adcsindH[i],wcc[wifiindex].VCC,wcc[wifiindex].MCP9700_Offset,wcc[wifiindex].MCP9700x);
        end;

m:=0;
{
OFFSET_WIFI=4;
STATUS_ADC_WIFI                 =0+OFFSET_WIFI;
STATUS_DIMM_WIFI                 =8+OFFSET_WIFI;
STATUS_OUT_WIFI                 =12+OFFSET_WIFI;
STATUS_INPUTS_WIFI              =13+OFFSET_WIFI;
STATUS_PROGRAM_NR_WIFI          =14+OFFSET_WIFI;
STATUS_ADC_PROGRAM_WIFI         =15+OFFSET_WIFI;
STATUS_MODE_WIFI=14+OFFSET_WIFI;
STATUS_MOD2_WIFI=15+OFFSET_WIFI;
STATUS_ADCL_WIFI=16+OFFSET_WIFI;
STATUS_ADCH_WIFI=24+OFFSET_WIFI;
STATUS_RSSI_WIFI=32+OFFSET_WIFI;
STATUS_QSI_WIFI=33+OFFSET_WIFI;
STATUS_SINCE_WIFI=34+OFFSET_WIFI;
 }
m:=0;
        for i:=0 to 7 do //bity
                begin
                if ((ord(statusb[+STATUS_OUT_WIFI+m]) and (1 shl i))>0)   then     //turned on output
                        begin
                        wcc[wifiindex].douts[m*8+i]:=true;
                        end
                else
                        begin
                        wcc[wifiindex].douts[m*8+i]:=false;
                        end;
                end;

//                end;
//offset:=0;
m:=0;
        for i:=0 to 7 do //bity
                begin
                if ((ord(statusb[STATUS_INPUTS_WIFI+m]) and (1 shl i))>0) then
                        begin
                        wcc[wifiindex].dins[m*8+i]:=true;
                        end
                else
                        wcc[wifiindex].dins[m*8+i]:=false;
                end;
//set some important information as program, adc program, program name, adc program name
wcc[wifiindex].currentprogram:=ord(statusb[STATUS_PROGRAM_NR_WIFI]) and $3f;
//wc[wifiindex].currentzone:=ord(statusb[STATUS_ZONE_NR_WIFI]);
wcc[wifiindex].currentadcprogram:=ord(statusb[STATUS_ADC_PROGRAM_WIFI]) and $3f;
wcc[wifiindex].rssi:=ord(statusb[STATUS_RSSI_WIFI]);
k:=0;
wcc[wifiindex].dimmers[k]:=ord(statusb[STATUS_DIMM_WIFI+k]);inc(k);
wcc[wifiindex].dimmers[k]:=ord(statusb[STATUS_DIMM_WIFI+k]);inc(k);
wcc[wifiindex].dimmers[k]:=ord(statusb[STATUS_DIMM_WIFI+k]);inc(k);
if (wcc[wifiindex].rssi>127) then wcc[wifiindex].rssi:=wcc[wifiindex].rssi-256;    //unsigned to signed conversion
wcc[wifiindex].qssi:=ord(statusb[STATUS_QSI_WIFI]);
wcc[wifiindex].since:=(ord(statusb[STATUS_SINCE_WIFI]) shl 8) + ord(statusb[STATUS_SINCE_WIFI+1]);

{//no programs available in eHWifi currently
if wc[wifiindex].currentprogram<XPROGRAMSW_COUNT then
        wc[wifiindex].currentprogramname:=wc[wifindex].programsnames[wc[wifiindex].currentprogram];
if wc[wifiindex].currentzone<XZONESW_COUNT then
        wc[wifiindex].currentzonename:=wc[wifiindex].zonenames[wc[wifiindex].currentzone];
if wc[wifiindex].currentadcprogram<XADCW_PROGRAMS_COUNT then
        wc[wifiindex].currentadcprogramname:=wifi[wifiindex].adcprogramsnames[wifi[wifiindex].currentadcprogram];
 }
end;



//////////////////////////////////////////////////////////////////////////////////

function calcindex(high,low:byte):integer;
begin
result:=(ord(high) and 3) shl 8;
result:=result+ord(low);
end;



end.
